﻿using System;
using System.IO;
using System.Collections.Generic;

namespace PureChat.Plugins.OnTime
{
	public sealed class Settings
	{
		public Settings(string configPath)
		{
			using (System.IO.FileStream fs = new System.IO.FileStream(configPath, System.IO.FileMode.Open, System.IO.FileAccess.Read))
			{
				using (System.Xml.XmlReader rdr = System.Xml.XmlTextReader.Create(fs))
				{
					string itemType = string.Empty;
					bool inDefaultValues = false;
					while (rdr.Read())
					{
						if (rdr.Name.Equals("settings", StringComparison.OrdinalIgnoreCase) || rdr.Name.Equals("OnTimePlugin", StringComparison.OrdinalIgnoreCase) || string.IsNullOrEmpty(rdr.Name))
							continue;

						if (rdr.Name.Equals("DefaultValues", StringComparison.OrdinalIgnoreCase))
							inDefaultValues = true;
						else if (rdr.Name.Equals("add", StringComparison.OrdinalIgnoreCase))
						{
							if (inDefaultValues)
								PopulateDefaultValues(rdr.GetAttribute("key"), rdr.GetAttribute("value"), itemType);
							else
								PopulateSetting(rdr.GetAttribute("key"), rdr.GetAttribute("value"));
						}
						else if (inDefaultValues && rdr.IsStartElement())
							itemType = rdr.Name.ToLower();
					}
				}
			}
		}

		private void PopulateSetting(string name, string value)
		{
			switch (name.ToLower())
			{
				case "sdkurl":
					SDKUrl = value;
					if (!SDKUrl.EndsWith("/"))
						SDKUrl += "/";
					break;
				case "securityguid":
					SecurityGuid = new Guid(value);
					break;
				case "ontimeweburl":
					OnTimeWebUrl = value;
					if (!string.IsNullOrEmpty(OnTimeWebUrl) && !OnTimeWebUrl.EndsWith("/"))
						OnTimeWebUrl += "/";
					break;
				case "htmldetailfields":
					if (!bool.TryParse(value, out HtmlDetailFields))
						HtmlDetailFields = true;
					break;
			}
		}

		private void PopulateDefaultValues(string name, string value, string itemType)
		{
			switch (itemType)
			{
				case "defect":
					DefectDefaultValues.Add(name, value);
					break;
				case "feature":
					FeatureDefaultValues.Add(name, value);
					break;
				case "task":
					TaskDefaultValues.Add(name, value);
					break;
				case "incident":
					IncidentDefaultValues.Add(name, value);
					break;
			}
		}

		public string SDKUrl = string.Empty;
		public Guid SecurityGuid;
		public string OnTimeWebUrl = string.Empty;
		public bool HtmlDetailFields = true;

		public Dictionary<string, string> DefectDefaultValues = new Dictionary<string, string>(4);
		public Dictionary<string, string> FeatureDefaultValues = new Dictionary<string, string>(4);
		public Dictionary<string, string> TaskDefaultValues = new Dictionary<string, string>(4);
		public Dictionary<string, string> IncidentDefaultValues = new Dictionary<string, string>(4);

		public string DefectServiceUrl
		{
			get { return SDKUrl + "defectservice.asmx"; }
		}
		public string FeatureServiceUrl
		{
			get { return SDKUrl + "featureservice.asmx"; }
		}
		public string TaskServiceUrl
		{
			get { return SDKUrl + "taskservice.asmx"; }
		}
		public string IncidentServiceUrl
		{
			get { return SDKUrl + "incidentservice.asmx"; }
		}
		public string ProjectServiceUrl
		{
			get { return SDKUrl + "projectservice.asmx"; }
		}
		public string CustomerServiceUrl
		{
			get { return SDKUrl + "customerservice.asmx"; }
		}
		public string UserServiceUrl
		{
			get { return SDKUrl + "userservice.asmx"; }
		}
		public string TypeServiceUrl
		{
			get { return SDKUrl + "typeservice.asmx"; }
		}
	}
}
